==> ol-2.2.tar.gz: ol-2.2/samples/bf.lisp <==
#!/usr/bin/env ol

; read char from stdin
(define (read)
   (let ((ch (syscall 0 stdin 0))) ; 0 means "is it has any symbols to read"
      (if (eq? ch #true) ; async io, read again
         (read)
         (ref ch 0))))


(define (bf program stack-length)
   (let ((program (string-append program "]"))
         (program-counter 0)
         (stack (make-bytevector stack-length 0))
         (stack-pointer 0))
      (letrec ((skip (lambda (PC SP)
                        (let loop ((pc PC) (sp SP))
                           (let ((ch (string-ref program pc))
                                 (pc (+ pc 1)))
                              (case ch
                                 (#\]  (list pc sp))
                                 (#\[  (apply loop (skip pc sp)))
                                 (else (loop pc sp)))))))
               (step (lambda (PC SP)
                        (let loop ((pc PC) (sp SP))
                           (let ((ch (string-ref program pc))
                                 (pc (+ pc 1)))
                              (case ch
                                 (#\]  (list (- PC 1) sp))
                                 (#\[  (if (eq? (ref stack sp) 0)
                                          (apply loop (skip pc sp))
                                          (apply loop (step pc sp))))
                                 (#\+  (set-ref! stack sp (+ (ref stack sp) 1))
                                       (loop pc sp))
                                 (#\-  (set-ref! stack sp (- (ref stack sp) 1))
                                       (loop pc sp))
                                 (#\>  (loop pc (+ sp 1)))
                                 (#\<  (loop pc (- sp 1)))
                                 (#\.  (display (make-string 1 (ref stack sp)))
                                       (loop pc sp))
                                 (#\,  (set-ref! stack sp (read))
                                       (loop pc sp))
                                 (else (loop pc sp))))))))
         (step 0 0))))

; use command line:
(if (zero? (length *vm-args*)) (begin
      (print "usage: bf.lisp \"bf-program\"")
      (halt 1)))
(setq program (list-ref *vm-args* 0))

; or use bf* self interpreter (short version)
;(setq program "
;>>>+[[-]>>[-]++>+>+++++++[<++++>>++<-]++>>+>+>+++++[>++>++++++<<-]+>>>,<++[[>[
;->>]<[>>]<<-]<[<]<+>>[>]>[<+>-[[<+>-]>]<[[[-]<]++<-[<+++++++++>[<->-]>>]>>]]<<
;]<]<[[<]>[[>]>>[>>]+[<<]<[<]<+>>-]>[>]+[->>]<<<<[[<<]<[<]+<<[+>+<<-[>-->+<<-[>
;+<[>>+<<-]]]>[<+>-]<]++>>-->[>]>>[>>]]<<[>>+<[[<]<]>[[<<]<[<]+[-<+>>-[<<+>++>-
;[<->[<<+>>-]]]<[>+<-]>]>[>]>]>[>>]>>]<<[>>+>>+>>]<<[->>>>>>>>]<<[>.>>>>>>>]<<[
;>->>>>>]<<[>,>>>]<<[>+>]<<[+<<]<]")

; or use bf* self interpreter (long version)
;(setq program "
;>>>>>+[->>++>+>+++++++[<++++>>++<-]++>>+>+>+++++[>++>++++++<<-]+>>>,<++[[>[->>
;]<[>>]<<-]<[<]<+>>[>]>[<+>-[[<+>-]>]<[[[-]<]++<-[<+++++++++>[<->-]>>]>>]]<<]>[
;-]+<<[--[[-]>>->+<<<]>>[-<<<<[>+<-]>>>>>>+<]<<]>>[-]<<>>>[<<<+>>>-]<<<]>>>+<<<
;<[<<]>>[[<+>>+<-]+<-[-[-[-[-[-[-[->->>[>>]>>[>>]<+<[<<]<<[<<]<]>[->>[>>]>>[>>]
;<,<[<<]<<[<<]]<]>[->>[>>]>>[>>]<-<[<<]<<[<<]]<]>[->>[>>]>>[>>]<.<[<<]<<[<<]]<]
;>[->>[>>]>>[>>]<<-<<[<<]<<[<<]]<]>[->>[>>]>>[>>]+[<<]<<[<<]]<]>[->>[>>]>>[>>]<
;[>+>>+<<<-]>[<+>-]>>[<<+>>[-]]+<<[>>-<<-]>>[<<+>>>>+<<-]>>[<<+>>-]<<[>>+<<-]+>
;>[<<->>-]<<<<[-<<[<<]<<[<<]<<<<<++>>]>>[-<<<<<<[<<]<<[<<]<]>]<]>[->>[>>]>>[>>]
;<[>+>>+<<<-]>[[<+>-]>>[-]+<<]>>[<<+>>>>+<<-]>>[<<+>>-]<<[>>+<<-]+>>[<<->>-]<<<
;<[-<<[<<]<<[<<]<<<<<+>>]>>[-<<<<<<[<<]<<[<<]<]>]>[<+>-]<<<<<<[>>+<<-[->>->>+[>
;>>[-<+>>+<]+<-[-[[-]>[-]<]>[-1<<<+>>>]<]>[-<<<->>>]>[-<+>]<<<<[>>+<<-]>>]<<<<<
;<]>>[-<<+[>>>[-<+>>+<]+<-[-[[-]>[-]<]>[-1<<<->>>]<]>[-<<<+>>>]>[-<+>]<<<<[<<+>
;>-]<<]]]>>>>>>>]")

; you can push onto bf interpreters next "hello world" line (don't forget exclamation mark)
;++++++++++[>+++++++>++++++++++>+++>+<<<<-]>++.>+.+++++++..+++.>++.<<+++++++++++++++.>.+++.------.--------.>+.>.!

(bf program 30000)
(halt 0)

; hello world: ++++++++++[>+++++++>++++++++++>+++>+<<<<-]>++.>+.+++++++..+++.>++.<<+++++++++++++++.>.+++.------.--------.>+.>.
; @: >>++++[<++++[<++++>-]>-]<<.[-]++++++++++.
; fibonacci numbers: +++++++++++>+>>>>++++++++++++++++++++++++++++++++++++++++++++>++++++++++++++++++++++++++++++++<<<<<<[>[>>
;                    >>>>+>+<<<<<<<-]>>>>>>>[<<<<<<<+>>>>>>>-]<[>++++++++++[-<-[>>+>+<<<-]>>>[<<<+>>>-]+<[>[-]<[-]]>[<<[>>>+<<
;                    <-]>>[-]]<<]>>>[>>+>+<<<-]>>>[<<<+>>>-]+<[>[-]<[-]]>[<<+>>[-]]<<<<<<<]>>>>>[+++++++++++++++++++++++++++++
;                    +++++++++++++++++++.[-]]++++++++++<[->-<]>++++++++++++++++++++++++++++++++++++++++++++++++.[-]<<<<<<<<<<<
;                    <[>>>+>+<<<<-]>>>>[<<<<+>>>>-]<-[>>.>.<<<[-]]<<[>>+>+<<<-]>>>[<<<+>>>-]<<[<+>-]>[<+>-]<<<-]

==> ol-2.3.7.tar.gz: ol-2.3.7/samples/bf.lisp <==
#!/usr/bin/env ol

; read char from stdin
(define (read)
   (let ((ch (syscall 0 stdin 0))) ; 0 means "is it has any symbols to read"
      (if (eq? ch #true) ; async io, read again
         (read)
         (ref ch 0))))


(define (bf program stack-length)
   (let ((program (string-append program "]"))
         (program-counter 0)
         (stack (make-bytevector stack-length 0))
         (stack-pointer 0))
      (letrec ((skip (lambda (PC SP)
                        (let loop ((pc PC) (sp SP))
                           (let ((ch (string-ref program pc))
                                 (pc (+ pc 1)))
                              (case ch
                                 (#\]  (list pc sp))
                                 (#\[  (apply loop (skip pc sp)))
                                 (else (loop pc sp)))))))
               (step (lambda (PC SP)
                        (let loop ((pc PC) (sp SP))
                           (let ((ch (string-ref program pc))
                                 (pc (+ pc 1)))
                              (case ch
                                 (#\]  (list (- PC 1) sp))
                                 (#\[  (if (eq? (ref stack sp) 0)
                                          (apply loop (skip pc sp))
                                          (apply loop (step pc sp))))
                                 (#\+  (set-ref! stack sp (+ (ref stack sp) 1))
                                       (loop pc sp))
                                 (#\-  (set-ref! stack sp (- (ref stack sp) 1))
                                       (loop pc sp))
                                 (#\>  (loop pc (+ sp 1)))
                                 (#\<  (loop pc (- sp 1)))
                                 (#\.  (display (make-string 1 (ref stack sp)))
                                       (loop pc sp))
                                 (#\,  (set-ref! stack sp (read))
                                       (loop pc sp))
                                 (else (loop pc sp))))))))
         (step 0 0))))

; use command line:
(if (zero? (length (command-line))) (begin
      (print "usage: bf.lisp \"bf-program\"")
      (halt 1)))
(setq program (list-ref (command-line) 0))

; or use bf* self interpreter (short version)
;(setq program "
;>>>+[[-]>>[-]++>+>+++++++[<++++>>++<-]++>>+>+>+++++[>++>++++++<<-]+>>>,<++[[>[
;->>]<[>>]<<-]<[<]<+>>[>]>[<+>-[[<+>-]>]<[[[-]<]++<-[<+++++++++>[<->-]>>]>>]]<<
;]<]<[[<]>[[>]>>[>>]+[<<]<[<]<+>>-]>[>]+[->>]<<<<[[<<]<[<]+<<[+>+<<-[>-->+<<-[>
;+<[>>+<<-]]]>[<+>-]<]++>>-->[>]>>[>>]]<<[>>+<[[<]<]>[[<<]<[<]+[-<+>>-[<<+>++>-
;[<->[<<+>>-]]]<[>+<-]>]>[>]>]>[>>]>>]<<[>>+>>+>>]<<[->>>>>>>>]<<[>.>>>>>>>]<<[
;>->>>>>]<<[>,>>>]<<[>+>]<<[+<<]<]")

; or use bf* self interpreter (long version)
;(setq program "
;>>>>>+[->>++>+>+++++++[<++++>>++<-]++>>+>+>+++++[>++>++++++<<-]+>>>,<++[[>[->>
;]<[>>]<<-]<[<]<+>>[>]>[<+>-[[<+>-]>]<[[[-]<]++<-[<+++++++++>[<->-]>>]>>]]<<]>[
;-]+<<[--[[-]>>->+<<<]>>[-<<<<[>+<-]>>>>>>+<]<<]>>[-]<<>>>[<<<+>>>-]<<<]>>>+<<<
;<[<<]>>[[<+>>+<-]+<-[-[-[-[-[-[-[->->>[>>]>>[>>]<+<[<<]<<[<<]<]>[->>[>>]>>[>>]
;<,<[<<]<<[<<]]<]>[->>[>>]>>[>>]<-<[<<]<<[<<]]<]>[->>[>>]>>[>>]<.<[<<]<<[<<]]<]
;>[->>[>>]>>[>>]<<-<<[<<]<<[<<]]<]>[->>[>>]>>[>>]+[<<]<<[<<]]<]>[->>[>>]>>[>>]<
;[>+>>+<<<-]>[<+>-]>>[<<+>>[-]]+<<[>>-<<-]>>[<<+>>>>+<<-]>>[<<+>>-]<<[>>+<<-]+>
;>[<<->>-]<<<<[-<<[<<]<<[<<]<<<<<++>>]>>[-<<<<<<[<<]<<[<<]<]>]<]>[->>[>>]>>[>>]
;<[>+>>+<<<-]>[[<+>-]>>[-]+<<]>>[<<+>>>>+<<-]>>[<<+>>-]<<[>>+<<-]+>>[<<->>-]<<<
;<[-<<[<<]<<[<<]<<<<<+>>]>>[-<<<<<<[<<]<<[<<]<]>]>[<+>-]<<<<<<[>>+<<-[->>->>+[>
;>>[-<+>>+<]+<-[-[[-]>[-]<]>[-1<<<+>>>]<]>[-<<<->>>]>[-<+>]<<<<[>>+<<-]>>]<<<<<
;<]>>[-<<+[>>>[-<+>>+<]+<-[-[[-]>[-]<]>[-1<<<->>>]<]>[-<<<+>>>]>[-<+>]<<<<[<<+>
;>-]<<]]]>>>>>>>]")

; you can push onto bf interpreters next "hello world" line (don't forget exclamation mark)
;++++++++++[>+++++++>++++++++++>+++>+<<<<-]>++.>+.+++++++..+++.>++.<<+++++++++++++++.>.+++.------.--------.>+.>.!

(bf program 30000)
(halt 0)

; hello world: ++++++++++[>+++++++>++++++++++>+++>+<<<<-]>++.>+.+++++++..+++.>++.<<+++++++++++++++.>.+++.------.--------.>+.>.
; @: >>++++[<++++[<++++>-]>-]<<.[-]++++++++++.
; fibonacci numbers: +++++++++++>+>>>>++++++++++++++++++++++++++++++++++++++++++++>++++++++++++++++++++++++++++++++<<<<<<[>[>>
;                    >>>>+>+<<<<<<<-]>>>>>>>[<<<<<<<+>>>>>>>-]<[>++++++++++[-<-[>>+>+<<<-]>>>[<<<+>>>-]+<[>[-]<[-]]>[<<[>>>+<<
;                    <-]>>[-]]<<]>>>[>>+>+<<<-]>>>[<<<+>>>-]+<[>[-]<[-]]>[<<+>>[-]]<<<<<<<]>>>>>[+++++++++++++++++++++++++++++
;                    +++++++++++++++++++.[-]]++++++++++<[->-<]>++++++++++++++++++++++++++++++++++++++++++++++++.[-]<<<<<<<<<<<
;                    <[>>>+>+<<<<-]>>>>[<<<<+>>>>-]<-[>>.>.<<<[-]]<<[>>+>+<<<-]>>>[<<<+>>>-]<<[<+>-]>[<+>-]<<<-]
